@@ -4,6 +4,7 @@ from StructNoSQL.models import DatabasePathElement
 from StructNoSQL.exceptions import FieldTargetNotFoundException
 from StructNoSQL.fields import BaseItem
 from StructNoSQL.practical_logger import message_with_vars
+from StructNoSQL.exceptions import MissingQueryKwarg
 
 MULTI_ATTRIBUTES_SELECTOR_REGEX_EXPRESSION = r'(\()(.*)(\))'
 
@@ -52,6 +53,8 @@ def process_and_get_field_path_object_from_field_path(field_path_key: str, field
     return _get_field_object_from_field_path(field_path_key=field_path_key, fields_switch=fields_switch), False
 
 def process_and_get_fields_paths_objects_from_fields_paths(fields_paths: List[str], fields_switch: dict) -> Dict[str, BaseItem]:
+    print("process_and_get_fields_paths_objects_from_fields_paths is deprecated and will soon be removed")
+    # todo: remove
     fields_objects_to_get = {}
     for field_key in fields_paths:
         field_path_object, has_multiple_fields_path = process_and_get_field_path_object_from_field_path(
@@ -86,14 +89,14 @@ def make_rendered_database_path(database_path_elements: List[DatabasePathElement
                         custom_default_value=path_element.custom_default_value
                     ))
                 else:
-                    raise Exception(message_with_vars(
+                    raise MissingQueryKwarg(message_with_vars(
                         message="A variable was required but not found in the query_kwargs dict passed to the make_rendered_database_path function.",
                         vars_dict={"keyVariableName": variable_name, "matchingKwarg": matching_kwarg,
                                    "queryKwargs": query_kwargs, "databasePathElements": database_path_elements}
                     ))
             else:
                 raise Exception(message_with_vars(
-                    message="A variable was required but not query_kwargs have been passed to the make_rendered_database_path function.",
+                    message="A variable was required but no query_kwargs have been passed to the make_rendered_database_path function.",
                     vars_dict={"keyVariableName": variable_name, "queryKwargs": query_kwargs, "databasePathElements": database_path_elements}
                 ))
     return output_database_path_elements
